from .credential import get_credential
from .format import class2str, dict2list, dict2str
from .logger import get_logger
from .types import CredentialType, InsertType, ScopeType, SlideLayout

__all__ = (
    "get_credential",
    "class2str",
    "dict2str",
    "dict2list",
    "get_logger",
    "CredentialType",
    "InsertType",
    "ScopeType",
    "SlideLayout",
)



import pkg_resources

from .api import SheetsAPI, SlidesAPI
from .utils import CredentialType, InsertType, ScopeType

__all__ = (
    "SheetsAPI",
    "SlidesAPI",
    "TEXT",
    "TABLE",
    "GRAPH",
    "IMAGE",
    "PE",
    "PR",
    "SE",
    "SR",
    "API_KEYS",
    "OAUTH",
    "SERVICE_ACCOUNT",
    "CredentialType",
    "InsertType",
    "ScopeType",
)

__version__ = pkg_resources.get_distribution("py2gsuite").version


# InsertType alias
TEXT = InsertType.TEXT
TABLE = InsertType.TABLE
GRAPH = InsertType.GRAPH
IMAGE = InsertType.IMAGE

# ScopeType alias
PE = ScopeType.PRESENTATION_EDITABLE
PR = ScopeType.PRESENTATION_READONLY
SE = ScopeType.SHEETS_EDITABLE
SR = ScopeType.SHEETS_READONLY

# CredentialType alias
API_KEYS = CredentialType.API_KEYS
OAUTH = CredentialType.OAUTH
SERVICE_ACCOUNT = CredentialType.SERVICE_ACCOUNT



import os.path as osp
import sys

from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow

from .logger import get_logger
from .types import CredentialType, ScopeType

logger = get_logger()


def get_credential(
    credential_file: str,
    credential_type: CredentialType = CredentialType.OAUTH,
    scope: ScopeType = ScopeType.DRIVE,
    **kwargs,
) -> Credentials:
    """[summary]
    Returns Credentials instance.

    Args:
        credential_file (str): The json file of credential.
        credential_type (CredentialType): The type of Credential.
            For Sheets/SlidesAPI only allows OAuth2.0 or API Keys. Defaults to CredentialType.OAUTH.
        scope (ScopeType): The type of scope. Defaults to ScopeType.DRIVE.

    **kwargs:
        host (str): Defaults to "localhost".
        port (int): Defaults to 8080.
        cache_token (bool): Defaults to False.

    Returns:
        creds (Credentials): Credential instance.

    Raises:
        FileNotFoundError: When cannot find specified credential file.
    """
    if not osp.exists(credential_file):
        raise FileNotFoundError(f"Cannot find {credential_file}")

    host: str = kwargs.get("host", "localhost")
    port: int = kwargs.get("port", 8080)
    cache_token: bool = kwargs.get("cache_token", False)

    creds: Credentials
    try:
        if credential_type == CredentialType.API_KEYS:
            # TODO
            raise NotImplementedError("Only support CredentialType.OAUTH")
        elif credential_type == CredentialType.OAUTH:
            flow: InstalledAppFlow = InstalledAppFlow.from_client_secrets_file(
                credential_file,
                scope.value,
            )
            creds: Credentials = flow.run_local_server(host=host, port=port)
        elif credential_type == CredentialType.SERVICE_ACCOUNT:
            raise ValueError("Only support OAuth2.0 or API keys")
        else:
            raise TypeError(f"`credential_type` must be an element of py2gsuite.Type, but got {credential_type}")
    except OSError as err:
        logger.error(err)
        sys.exit(1)

    if cache_token:
        logger.warn("Important personal info is included in the token.json. Please keep it safe.")
        with open(osp.join(osp.dirname(__file__), "token.json"), "w") as token:
            token.write(creds.to_json())

    return creds


def get_credential_from_token(scope: ScopeType):
    """[summary]
    Get Credentials instance from cached token.json.

    Args:
        scope (ScopeType): The ScopeType instance.

    Returns:
        creds (Credentials): The Credentials instance loaded from token.json.

    Raises:
        FileNotFoundError: When there is no token.json.
    """
    if not osp.exists(osp.basename(__file__), "token.json"):
        raise FileNotFoundError("Cannot find token.json")

    try:
        creds: Credentials = Credentials.from_authorized_user_file(
            "token.json",
            scopes=scope,
        )
    except OSError as err:
        logger.error(err)
        sys.exit(1)

    return creds



from __future__ import annotations

from secrets import token_hex
from typing import Any, Dict, List, Optional

from google.oauth2.credentials import Credentials
from googleapiclient.discovery import Resource, build
from googleapiclient.errors import HttpError

from py2gsuite.utils import SlideLayout, get_logger

from .base import APIBase

__all__ = ["SlidesAPI"]

logger = get_logger()


class SlidesAPI(APIBase):
    """The wrapper of Google Slides API.

    Attributes:
        creds (Credentials): The Credentials instance.
        id (str): The ID of presentation.
        service (Resource): The Resource instance.
    """

    def __init__(
        self,
        creds: Credentials,
        presentation_id: str,
        service: Optional[Resource] = None,
    ) -> None:
        """
        Args:
            creds (Credentials): Credentials instance.
            presentation_id (str): ID of presentation.
            service (Optional[Resource]): Resource instance to connect to spreadsheet.
                Defaults to None.
        """
        super().__init__(creds=creds, file_id=presentation_id)
        if service is None:
            self.service: Resource = build("slides", "v1", credentials=self.creds)
        else:
            assert hasattr(service, "presentations")
            self.service: Resource = service

    @classmethod
    def with_new(cls, creds: Credentials, title: str) -> Optional[SlidesAPI]:
        """[summary]
        Create SlidesAPI instance with new presentation.

        Args:
            creds (Credentials): Credentials instance.
            title (str): The title of presentation.

        Returns:
            Optional[SlidesAPI]: SlidesAPI instance. If fail, returns None.
        """
        try:
            service = build("slides", "v1", credentials=creds)
            body = {"title": title}
            presentation = service.presentations().create(body=body).execute()
            presentation_id: str = presentation.get("presentationId")
            logger.info(f"Created presentation with ID:" f"{presentation_id}")
        except HttpError as err:
            logger.error(err)
            return None

        return cls(creds=creds, presentation_id=presentation_id, service=service)

    def __post_update(self, requests: List[Any]) -> Optional[Dict[str, Any]]:
        """[summary]
        Post update requests.

        Args:
            requests (List[Any]): Requests to be posted.

        Returns:
            response (Optional[Dict[str, Any]]): Response result as dict. If fail, returns None.
        """
        try:
            body: Dict[str, List[Any]] = {"requests": requests}
            response: Dict[str, Any] = (
                self.service.presentations()
                .batchUpdate(
                    presentationId=self.id,
                    body=body,
                )
                .execute()
            )
        except HttpError as err:
            logger.error(err)
            return None

        return response

    def exists_page(self, page_id: str) -> bool:
        """Check if the page that has specified page_id exists.

        Args:
            page_id (str): ID of page.

        Returns:
            bool: Wether page exists.
        """
        try:
            response_id: Optional[str] = (
                self.service.presentations()
                .pages()
                .get(
                    presentationId=self.id,
                    pageObjectId=page_id,
                )
            )
        except HttpError as err:
            logger.error(err)
            return False

        return True if response_id == page_id else False

    def create_slide(
        self,
        page_id: str,
        layout: SlideLayout = SlideLayout.BLANK,
    ) -> bool:
        """[summary]
        Create new slide to the presentation.

        Args:
            page_id (str): ID of new page.
            layout (SlideLayout): Defaults to SlideLayout.BLANK

        Returns:
            bool: Whether succeeded to create new slide.
        """
        # if self.exists_page(page_id):
        #     return True
        requests = [
            {
                "createSlide": {
                    "objectId": page_id,
                    "insertionIndex": "1",
                    "slideLayoutReference": {
                        "predefinedLayout": layout.value,
                    },
                }
            }
        ]
        response = self.__post_update(requests)
        if response is not None:
            create_slide_response = response.get("replies")[0].get("createSlide")
            logger.info(f"Created slide with ID:" f"{(create_slide_response.get('objectId'))}")
            return True
        return False

    def add_text(self, text: str, page_id: Optional[str] = None, **kwargs) -> bool:
        """[summary]
        Add new text to specified slide.
        If page_id is not specified, add text to the first slide.

        Args:
            text (str): text info to be inserted.
            element_id (str): ID of text box.
            page_id (str): page ID to be inserted.

        **kwargs:
            element_id (str): the element ID of text. Defaults to 'NewTextBox'.
            magnitude (int): magnitude of textbox. Defaults to 350.

        Returns:
            bool: Whether succeeded to add text.
        """
        if page_id is None:
            page_id = "p"

        element_id: str = kwargs.get("element_id", token_hex(16))
        pt: Dict[str, Any] = {"magnitude": kwargs.get("magnitude", 100), "unit": "PT"}

        requests: List[Dict[str, Any]] = [
            {
                "createShape": {
                    "objectId": element_id,
                    "shapeType": "TEXT_BOX",
                    "elementProperties": {
                        "pageObjectId": page_id,
                        "size": {"height": pt, "width": pt},
                        "transform": {"scaleX": 1, "scaleY": 1, "translateX": 350, "translateY": 100, "unit": "PT"},
                    },
                }
            },
            # Insert text into the box, using the supplied element ID.
            {"insertText": {"objectId": element_id, "insertionIndex": 0, "text": text}},
        ]
        response = self.__post_update(requests)
        if response is not None:
            create_shape_response = response.get("replies")[0].get("createShape")
            logger.info(f"Created textbox with ID: {create_shape_response.get('objectId')}")
            return True
        return False

    def add_image(self, img_url: str, page_id: Optional[str] = None, **kwargs) -> bool:
        """[summary]
        Add new image to specified slide.
        If page_id is not specified, add image to the first slide.

        Args:
            img_url (str): URL of image.
            page_id (Optional[str]): ID of page. Defaults to None.

        **kwargs:
            image_id (str): ID of image. If None, create with random token. Defaults to None.
            magnitude (int): Size of image. Defaults to 4000.

        Returns:
            bool: Whether succeeded to add image.
        """
        if page_id is None:
            page_id = "p"

        image_id: str = kwargs.get("image_id", token_hex(16))
        emu: Dict[str, Any] = {"magnitude": kwargs.get("magnitude", 4000), "unit": "EMU"}
        requests: List[Dict[str, Any]] = [
            {
                "createImage": {
                    "objectId": image_id,
                    "url": img_url,
                    "elementProperties": {
                        "pageObjectId": page_id,
                        "size": {"height": emu, "width": emu},
                        "transform": {
                            "scaleX": 1,
                            "scaleY": 1,
                            "translateX": 100000,
                            "translateY": 100000,
                            "unit": "EMU",
                        },
                    },
                }
            }
        ]
        response: Optional[Dict[str, Any]] = self.__post_update(requests)
        if response is not None:
            create_image_response = response.get("replies")[0].get("createImage")
            logger.info(f"Created image with ID: {create_image_response.get('objectId')}")
            return True
        return False

    def create_empty_table(self, table_id: str, rows: int, cols: int, page_id: Optional[str] = None) -> bool:
        """Create empty table.

        Args:
            table_id (str): The ID of table.
            rows (int): The number of rows.
            cols (int): The number of columns.
            page_id (Optional[str]): The ID of page. If None, the table will be created on the first page.
                Defaults to None.

        Returns:
            bool: Whether succeeded to create the table.
        """
        if page_id is None:
            page_id = "p"

        requests: List[Dict[str, Any]] = [
            {
                "createTable": {
                    "objectId": table_id,
                    "elementProperties": {
                        "pageObjectId": page_id,
                    },
                    "rows": rows,
                    "cols": cols,
                }
            }
        ]
        response: Optional[Dict[str, Any]] = self.__post_update(requests)
        if response is not None:
            create_table_response = response.get("replies")[0].get("createTable")
            logger.info(f"Created table with ID: {create_table_response.get('objectId')}")
            return True
        return False

    def add_table(
        self,
        values: List[List[str]],
        table_id: Optional[str] = None,
        page_id: Optional[str] = None,
    ) -> bool:
        """Add values to the table.

        Args:
            values (List[List[str]]): Values of elements, in shape (rows, cols).
            table_id (Optional[str]): ID of table. If None, create new table. Defaults to None.
            page_id (Optional[str]): ID of page. If None, create on the first page. Defaults to None.

        Returns:
            bool: Whether succeeded to add elements in the table.
        """
        assert isinstance(values, list)
        assert all([isinstance(e, list) for e in values])

        rows: int = len(values)
        cols: int = len(values[0])
        assert all([len(e) == cols for e in values])

        # Create new empty table
        if table_id is None:
            table_id: str = token_hex(16)
            assert self.create_empty_table(
                table_id=table_id,
                rows=rows,
                cols=cols,
                page_id=page_id,
            )

        # Insert to table
        requests: List[Dict[str, Any]] = []
        # cnt: int = 0
        for i, v_rows in enumerate(values):
            for j, v in enumerate(v_rows):
                requests.append(
                    {
                        "insertText": {
                            "objectId": table_id,
                            "cellLocation": {
                                "rowindex": i,
                                "columnIndex": j,
                            },
                            "text": str(v),
                            "insertionIndex": 0,  # cnt
                        }
                    }
                )
                # cnt += 1
        response: Optional[Dict[str, Any]] = self.__post_update(requests)
        if response is not None:
            return True
        return False



from __future__ import annotations

from typing import Any, Dict, List, Optional

from google.oauth2.credentials import Credentials
from googleapiclient.discovery import Resource, build
from googleapiclient.errors import HttpError

from py2gsuite.utils import get_logger

from .base import APIBase

__all__ = ["SheetsAPI"]

logger = get_logger()


class SheetsAPI(APIBase):
    """[summary]
    The wrapper of Google Sheets API.

    Attributes:
        creds (Credentials): Credentials instance.
        sheet_id (str): ID of spreadsheet.
        service (Resource): Resource instance to connect to spreadsheet.
    """

    def __init__(
        self,
        creds: Credentials,
        sheet_id: str,
        service: Optional[Resource] = None,
    ) -> None:
        """
        Args:
            creds (Credentials): Credentials instance.
            sheet_id (str): ID of spreadsheet.
            service (Optional[Resource]): Resource instance to connect to spreadsheet.
                Defaults to None.
        """
        super().__init__(creds=creds, file_id=sheet_id)
        if service is None:
            self.service: Resource = build("sheets", "v4", credentials=creds)
        else:
            assert hasattr(service, "spreadsheets")
            self.service: Resource = service

    @classmethod
    def with_new(cls, creds: Credentials, title: str) -> Optional[SheetsAPI]:
        """Create instance with a new sheet.

        Args:
            creds (Credentials): The Credentials instance.
            title (str): The title of a sheet.

        Returns:
            Optional[SheetsAPI]: If failed to request, returns None.
        """
        try:
            service: Resource = build("sheets", "v4", credentials=creds)
            body = {"properties": {"title": title}}
            spreadsheet = service.spreadsheets().create(body=body, fields="spreadsheetId").execute()
            sheet_id: str = spreadsheet.get("spreadsheetId")
            logger.info(f"Spreadsheet ID: {sheet_id}")
        except HttpError as err:
            logger.error(err)
            return None
        return cls(creds, sheet_id, service)

    def add_values(
        self,
        values: List[List[str]],
        range_name: str,
        value_input_option: Optional[str] = None,
        sheet_id: Optional[str] = None,
    ) -> bool:
        """Add values on the cells. If cells are already filled, the old ones are remained.

        Args:
            values (List[List[str]]): Values of cells, in shape (rows, cols)
            range_name (str): Range of cells.
                For example, 'A1:C2' means values will be inserted on the cells from A1 to B2.
            value_input_option (Optional[str]): Input option. Defaults to None.
            sheet_id (Optional[str]): ID of sheet. Defaults to None.

        Returns:
            bool: Whether succeeded to add values.
        """
        if value_input_option is None:
            value_input_option = "USER_ENTERED"

        if sheet_id is None:
            sheet_id = self.id

        try:
            body: Dict[str, List[Any]] = {"values": values}
            result: Dict[str, Any] = (
                self.service.spreadsheets()
                .values()
                .append(
                    spreadsheetId=sheet_id,
                    range=range_name,
                    valueInputOption=value_input_option,
                    body=body,
                )
                .execute()
            )
            logger.info(f"{(result.get('updates').get('updatedCells'))} cells appended.")
        except HttpError as err:
            logger.error(err)
            return False

        return True

    def update_values(
        self,
        values: List[List[str]],
        range_name: str,
        value_input_option: Optional[str] = None,
        sheet_id: Optional[str] = None,
    ) -> bool:
        """Add values on the cells. If cells are already filled, these will be overwritten.

        Args:
            values (List[List[str]]): Values of cells, in shape (rows, cols)
            range_name (str): Range of cells.
                For example, 'A1:C2' means values will be inserted on the cells from A1 to B2.
            value_input_option (Optional[str]): Input option. Defaults to None.
            sheet_id (Optional[str]): ID of sheet. Defaults to None.

        Returns:
            bool: Whether succeeded to update values.
        """
        if value_input_option is None:
            value_input_option = "USER_ENTERED"

        if sheet_id is None:
            sheet_id = self.id

        try:
            body: Dict[str, List[Any]] = {"values": values}
            result: Dict[str, Any] = (
                self.service.spreadsheets()
                .values()
                .update(
                    spreadsheetId=sheet_id,
                    range=range_name,
                    valueInputOption=value_input_option,
                    body=body,
                )
                .execute()
            )
            logger.info(f"{result.get('updatedCells')} cells updated.")
        except HttpError as err:
            logger.error(err)
            return False

        return True

    def is_empty(self, range_name: str) -> bool:
        """Check whether specified cells are empty.
        Args:
            range_name (str): Range of cells.

        Returns:
            bool: Whether all cells are empty.
        """
        result = self.service.spreadsheets().values().get(spreadsheetId=self.id, range=range_name).execute()
        values = result.get("values")

        return values is None



from enum import Enum


class InsertType(Enum):
    """Type of inserted in google suite."""

    TEXT = "text"
    TABLE = "table"
    GRAPH = "graph"
    IMAGE = "image"


class ScopeType(Enum):
    """Type of scopes for google suite.

    PRESENTATION_EDITABLE: Allows read/write access to the user's presentations and their properties.
    PRESENTATION_READONLY: Allows read-only access to the user's presentations and their properties.
    SHEETS_EDITABLE: Allows read/write access to the user's sheets and their properties.
    SHEETS_READONLY: Allows read-only access to the user's sheets and their properties.
    DRIVE_FILES: Per-file access to files created or opened by the app.
    DRIVE_READONLY: Allows read-only access to the user's file metadata and file content.
    DRIVE: Full, permissive scope to access all of a user's files.
        Request this scope only when it is strictly necessary.
    """

    PRESENTATION_EDITABLE = ["https://www.googleapis.com/auth/presentations"]
    PRESENTATION_READONLY = ["https://www.googleapis.com/auth/presentations.readonly"]
    SHEETS_EDITABLE = ["https://www.googleapis.com/auth/spreadsheets"]
    SHEETS_READONLY = ["https://www.googleapis.com/auth/spreadsheets.readonly"]
    DRIVE_FILES = ["https://www.googleapis.com/auth/drive.file"]
    DRIVE_READONLY = ["https://www.googleapis.com/auth/drive.readonly"]
    DRIVE = ["https://www.googleapis.com/auth/drive"]


class CredentialType(Enum):
    """Type of credential."""

    API_KEYS = "api_keys"
    OAUTH = "oauth"
    SERVICE_ACCOUNT = "service_account"


class SlideLayout(Enum):
    """Type of slide layout"""

    BLANK = "BLANK"
    CAPTION_ONLY = "CAPTION_ONLY"
    TITLE = "TITLE"
    TITLE_AND_BODY = "TITLE_AND_BODY"
    TITLE_AND_TWO_COLUMNS = "TITLE_AND_TWO_COLUMNS"
    TITLE_ONLY = "TITLE_ONLY"
    SECTION_HEADER = "SECTION_HEADER"
    SECTION_TITLE_AND_DESCRIPTION = "SECTION_TITLE_AND_DESCRIPTION"
    ONE_COLUMN_TEXT = "ONE_COLUMN_TEXT"
    MAIN_POINT = "MAIN_POINT"
    BIG_NUMBER = "BIG_NUMBER"



import pprint
from enum import Enum
from typing import Any, Dict, List, Optional


def class2str(
    obj: object,
    abbreviation: Optional[int] = None,
    format: bool = False,
) -> str:
    """Convert class object to str.

    NOTE:
        Reference is below.
        https://stackoverflow.com/questions/1036409/recursively-convert-python-object-graph-to-dictionary

    Args:
        obj (object): Class object which you want to convert to str.
        abbreviation (Optional[int]): If len(list_object) > abbreviation, abbreviation the result.

    Returns:
        str: str converted from class object.
    """
    return dict2str(class2dict(obj, abbreviation), format=format)


def class2dict(
    obj: object,
    abbreviation: Optional[int] = None,
    class_key: Optional[str] = None,
) -> Dict[str, Any]:
    """[summary]

    Args:
        obj (object): Class object which you want to convert to dict.
        abbreviation (Optional[int]): If len(list_object) > abbreviation, abbreviate the result. Defaults to None.
        class_key (Optional[str]): Class key for dict. Defaults to None.
    """
    if isinstance(obj, dict):
        data: Dict[str, Any] = {}
        for key, item in obj.items():
            data[key] = class2dict(item, abbreviation, class_key)
        return data
    elif isinstance(obj, Enum):
        return str(obj)
    elif hasattr(obj, "_ast"):
        return class2dict(obj._ast(), abbreviation)
    elif hasattr(obj, "__iter__") and not isinstance(obj, str):
        if abbreviation and len(obj) > abbreviation:
            return f" --- length of element {len(object)} ---,"
        return [class2dict(elem, abbreviation, class_key) for elem in obj]
    elif hasattr(obj, "__dict__"):
        data: Dict[str, Any] = dict(
            [
                (key, class2dict(item, abbreviation, class_key))
                for key, item in obj.__dict__.items()
                if not callable(item) and not key.startswith("_")
            ]
        )
        if class_key is not None and hasattr(obj, "__class__"):
            data[class_key] = obj.__class__.__name__
        return data
    else:
        return obj


def dict2str(dict_obj: Dict[str, Any], format: bool = False) -> str:
    """[str]
    Convert dict object to str.

    Args:
        dict_obj (Dict[str, Any])
        format (bool): Whether format converted str. Defaults to False.

    Returns:
        str_ (str)
    """
    str_: str = pprint.pformat(
        dict_obj,
        indent=1,
        width=120,
        depth=None,
        compact=True,
    )
    if format:
        return "\n" + str_ + "\n"
    return str_


def dict2list(
    dict_obj: Dict[str, Any],
    keys: Optional[List[str]] = None,
) -> List[str]:
    """Convert dict to list of str.

    Args:
        dict_obj (Dict[str, Any])
        keys (Optional[List[str]]): Defaults to None.

    Returns:
        List[str]:
    """
    if keys is not None:
        ret: List[str] = []
        for k in keys:
            ret.append(str(dict_obj[k]))
        return ret

    return list(_get_values(dict_obj))


def _get_values(d: Dict[str, Any]):
    for v in d.values():
        if isinstance(v, dict):
            yield from dict2list(v)
        else:
            yield str(v)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['py2gsuite', 'py2gsuite.api', 'py2gsuite.utils']

package_data = \
{'': ['*']}

install_requires = \
['coloredlogs>=15.0.1,<16.0.0',
 'google-api-python-client>=2.52.0,<3.0.0',
 'google-auth-httplib2>=0.1.0,<0.2.0',
 'google-auth-oauthlib>=0.5.2,<0.6.0']

setup_kwargs = {
    'name': 'py2gsuite',
    'version': '0.2.0',
    'description': '',
    'long_description': None,
    'author': 'ktro2828',
    'author_email': 'kotaro.uetake@tier4.jp',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from abc import ABC, abstractclassmethod, abstractmethod
from typing import Optional

from google.oauth2.credentials import Credentials
from googleapiclient.discovery import Resource


class APIBase(ABC):
    @abstractmethod
    def __init__(self, creds: Credentials, file_id: str) -> None:
        super().__init__()
        self.creds: Credentials = creds
        self.id: str = file_id
        self.service: Resource

    @classmethod
    @abstractclassmethod
    def with_new(cls, creds: Credentials, title: str) -> Optional[APIBase]:
        pass

    def close(self) -> None:
        self.service.close()

    def __del__(self):
        self.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.__del__()



import logging

import coloredlogs


def get_logger(name=__name__, level=logging.INFO):
    """Returns logger.

    Args:
        name (str): module name.
        level (int): logging level.

    Returns:
        logger (logging.RootLogger): logger.
    """
    logger = logging.getLogger(name)
    logger.handlers.clear()
    logger.propagate = False
    logger.setLevel(level)

    # log handler
    handler = logging.StreamHandler()
    handler.setLevel(level)

    # formatter
    formatter = coloredlogs.ColoredFormatter(
        fmt="[%(asctime)s] [%(levelname)s] [func] %(funcName)s [line] %(lineno)d: %(message)s",
        datefmt="%Y-%d-%d %H:%M:%S",
        level_styles={
            "critical": {"color": "red", "bold": True},
            "error": {"color": "red"},
            "warning": {"color": "yellow"},
            "notice": {"color": "magenta"},
            "info": {},
            "debug": {"color": "green"},
            "spam": {"color": "green", "faint": True},
            "success": {"color": "green", "bold": True},
            "verbose": {"color": "blue"},
        },
        field_styles={
            "asctime": {"color": "green"},
            "levelname": {"color": "cyan", "bold": True},
            "funcName": {"color": "blue"},
            "lineno": {"color": "blue", "bold": True},
        },
    )

    handler.setFormatter(formatter)
    logger.addHandler(handler)

    return logger
